@import "~nib/index.styl"

.g-codemirror-edit-container
  margin-top 20px

.li-item-view-codemirror
  .editor
    position relative
    height 500px
    width 100%

.CodeMirror
  width 100%
  height 100%
  border 1px solid grey

.li-item-header-btn-group
  margin-top -3px
